@@ -1,38 +1,55 @@
 # -*- coding: utf-8 -*-
+#    Copyright (C) 2006-2018 by
+#    Aric Hagberg <hagberg@lanl.gov>
+#    Dan Schult <dschult@colgate.edu>
+#    Pieter Swart <swart@lanl.gov>
+#    All rights reserved.
+#    BSD license.
+#
+# Authors:
+#    Aric Hagberg <aric.hagberg@gmail.com>
+#    Dan Schult <dschult@colgate.edu>
+#    Ben Edwards <bedwards@cs.unm.edu>
+#    Neil Girdhar <neil.girdhar@mcgill.ca>
+#
 """Algorithms for directed acyclic graphs (DAGs).
 
 Note that most of these functions are only guaranteed to work for DAGs.
 In general, these functions do not check for acyclic-ness, so it is up
 to the user to check for that.
 """
-#    Copyright (C) 2006-2017 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
+
+from collections import defaultdict, deque
 from fractions import gcd
+from functools import partial
+from itertools import chain
+from itertools import product
+from itertools import starmap
 import heapq
 
 import networkx as nx
-from networkx.utils import consume, arbitrary_element, pairwise
-from networkx.utils.decorators import *
+from networkx.generators.trees import NIL
+from networkx.utils import arbitrary_element
+from networkx.utils import consume
+from networkx.utils import pairwise
+from networkx.utils import generate_unique_node
+from networkx.utils import not_implemented_for
 
-__author__ = """\n""".join(['Aric Hagberg <aric.hagberg@gmail.com>',
-                            'Dan Schult (dschult@colgate.edu)',
-                            'Ben Edwards (bedwards@cs.unm.edu)',
-                            'Neil Girdhar (neil.girdhar@mcgill.ca)'])
 __all__ = ['descendants',
            'ancestors',
            'topological_sort',
            'lexicographical_topological_sort',
+           'all_topological_sorts',
            'is_directed_acyclic_graph',
            'is_aperiodic',
            'transitive_closure',
            'transitive_reduction',
            'antichains',
            'dag_longest_path',
-           'dag_longest_path_length']
+           'dag_longest_path_length',
+           'dag_to_branching']
+
+chaini = chain.from_iterable
 
 
 def descendants(G, source):
@@ -51,9 +68,8 @@ def descendants(G, source):
     """
     if not G.has_node(source):
         raise nx.NetworkXError("The node %s is not in the graph." % source)
-    spl = nx.shortest_path_length(G, source=source)
-    des = set(spl) - set([source])
-    return des
+    des = set(n for n, d in nx.shortest_path_length(G, source=source).items())
+    return des - {source}
 
 
 def ancestors(G, source):
@@ -72,9 +88,18 @@ def ancestors(G, source):
     """
     if not G.has_node(source):
         raise nx.NetworkXError("The node %s is not in the graph." % source)
-    spl = nx.shortest_path_length(G, target=source)
-    anc = set(spl) - set([source])
-    return anc
+    anc = set(n for n, d in nx.shortest_path_length(G, target=source).items())
+    return anc - {source}
+
+
+def has_cycle(G):
+    """Decides whether the directed graph has a cycle."""
+    try:
+        consume(topological_sort(G))
+    except nx.NetworkXUnfeasible:
+        return True
+    else:
+        return False
 
 
 def is_directed_acyclic_graph(G):
@@ -90,13 +115,7 @@ def is_directed_acyclic_graph(G):
     bool
         True if `G` is a DAG, False otherwise
     """
-    if not G.is_directed():
-        return False
-    try:
-        consume(topological_sort(G))
-        return True
-    except nx.NetworkXUnfeasible:
-        return False
+    return G.is_directed() and not has_cycle(G)
 
 
 def topological_sort(G):
@@ -125,7 +144,7 @@ def topological_sort(G):
     NetworkXUnfeasible
         If `G` is not a directed acyclic graph (DAG) no topological sort exists
         and a :exc:`NetworkXUnfeasible` exception is raised.  This can also be
-        raised if `G` is changed while the returned iterator is being processed.
+        raised if `G` is changed while the returned iterator is being processed
 
     RuntimeError
         If `G` is changed while the returned iterator is being processed.
@@ -138,6 +157,16 @@ def topological_sort(G):
     >>> list(reversed(list(nx.topological_sort(DG))))
     [3, 2, 1]
 
+    If your DiGraph naturally has the edges representing tasks/inputs
+    and nodes representing people/processes that initiate tasks, then
+    topological_sort is not quite what you need. You will have to change
+    the tasks to nodes with dependence reflected by edges. The result is
+    a kind of topological sort of the edges. This can be done
+    with :func:`networkx.line_graph` as follows:
+
+    >>> list(nx.topological_sort(nx.line_graph(DG)))
+    [(1, 2), (2, 3)]
+
     Notes
     -----
     This algorithm is based on a description and proof in
@@ -149,8 +178,8 @@ def topological_sort(G):
 
     References
     ----------
-    .. [1] Manber, U. (1989). "Introduction to Algorithms: A Creative Approach."
-       Addison-Wesley.
+    .. [1] Manber, U. (1989).
+       *Introduction to Algorithms - A Creative Approach.* Addison-Wesley.
     """
     if not G.is_directed():
         raise nx.NetworkXError(
@@ -211,7 +240,7 @@ def lexicographical_topological_sort(G, key=None):
     NetworkXUnfeasible
         If `G` is not a directed acyclic graph (DAG) no topological sort exists
         and a :exc:`NetworkXUnfeasible` exception is raised.  This can also be
-        raised if `G` is changed while the returned iterator is being processed.
+        raised if `G` is changed while the returned iterator is being processed
 
     RuntimeError
         If `G` is changed while the returned iterator is being processed.
@@ -227,15 +256,15 @@ def lexicographical_topological_sort(G, key=None):
 
     References
     ----------
-    .. [1] Manber, U. (1989). "Introduction to Algorithms: A Creative Approach."
-       Addison-Wesley.
+    .. [1] Manber, U. (1989).
+       *Introduction to Algorithms - A Creative Approach.* Addison-Wesley.
     """
     if not G.is_directed():
         raise nx.NetworkXError(
             "Topological sort not defined on undirected graphs.")
 
     if key is None:
-        key = lambda x: x
+        def key(x): return x
 
     def create_tuple(node):
         return key(node), node
@@ -266,6 +295,126 @@ def lexicographical_topological_sort(G, key=None):
                                     "during iteration")
 
 
+@not_implemented_for('undirected')
+def all_topological_sorts(G):
+    """Returns a generator of _all_ topological sorts of the directed graph G.
+
+    A topological sort is a nonunique permutation of the nodes such that an
+    edge from u to v implies that u appears before v in the topological sort
+    order.
+
+    Parameters
+    ----------
+    G : NetworkX DiGraph
+        A directed graph
+
+    Returns
+    -------
+    generator
+        All topological sorts of the digraph G
+
+    Raises
+    ------
+    NetworkXNotImplemented
+        If `G` is not directed
+    NetworkXUnfeasible
+        If `G` is not acyclic
+
+    Examples
+    --------
+    To enumerate all topological sorts of directed graph:
+
+    >>> DG = nx.DiGraph([(1, 2), (2, 3), (2, 4)])
+    >>> list(nx.all_topological_sorts(DG))
+    [[1, 2, 4, 3], [1, 2, 3, 4]]
+
+    Notes
+    -----
+    Implements an iterative version of the algorithm given in [1].
+
+    References
+    ----------
+    .. [1] Knuth, Donald E., Szwarcfiter, Jayme L. (1974).
+       "A Structured Program to Generate All Topological Sorting Arrangements"
+       Information Processing Letters, Volume 2, Issue 6, 1974, Pages 153-157,
+       ISSN 0020-0190,
+       https://doi.org/10.1016/0020-0190(74)90001-5.
+       Elsevier (North-Holland), Amsterdam
+    """
+    if not G.is_directed():
+        raise nx.NetworkXError(
+            "Topological sort not defined on undirected graphs.")
+
+    # the names of count and D are chosen to match the global variables in [1]
+    # number of edges originating in a vertex v
+    count = dict(G.in_degree())
+    # vertices with indegree 0
+    D = deque([v for v, d in G.in_degree() if d == 0])
+    # stack of first value chosen at a position k in the topological sort
+    bases = []
+    current_sort = []
+
+    # do-while construct
+    while True:
+        assert all([count[v] == 0 for v in D])
+
+        if len(current_sort) == len(G):
+            yield list(current_sort)
+
+            # clean-up stack
+            while len(current_sort) > 0:
+                assert len(bases) == len(current_sort)
+                q = current_sort.pop()
+
+                # "restores" all edges (q, x)
+                # NOTE: it is important to iterate over edges instead
+                # of successors, so count is updated correctly in multigraphs
+                for _, j in G.out_edges(q):
+                    count[j] += 1
+                    assert count[j] >= 0
+                # remove entries from D
+                while len(D) > 0 and count[D[-1]] > 0:
+                    D.pop()
+
+                # corresponds to a circular shift of the values in D
+                # if the first value chosen (the base) is in the first
+                # position of D again, we are done and need to consider the
+                # previous condition
+                D.appendleft(q)
+                if D[-1] == bases[-1]:
+                    # all possible values have been chosen at current position
+                    # remove corresponding marker
+                    bases.pop()
+                else:
+                    # there are still elements that have not been fixed
+                    # at the current position in the topological sort
+                    # stop removing elements, escape inner loop
+                    break
+
+        else:
+            if len(D) == 0:
+                raise nx.NetworkXUnfeasible("Graph contains a cycle.")
+
+            # choose next node
+            q = D.pop()
+            # "erase" all edges (q, x)
+            # NOTE: it is important to iterate over edges instead
+            # of successors, so count is updated correctly in multigraphs
+            for _, j in G.out_edges(q):
+                count[j] -= 1
+                assert count[j] >= 0
+                if count[j] == 0:
+                    D.append(j)
+            current_sort.append(q)
+
+            # base for current position might _not_ be fixed yet
+            if len(bases) < len(current_sort):
+                bases.append(q)
+
+        if len(bases) == 0:
+            break
+
+
 def is_aperiodic(G):
     """Return True if `G` is aperiodic.
 
@@ -354,9 +503,7 @@ def transitive_closure(G):
     .. [1] http://www.ics.uci.edu/~eppstein/PADS/PartialOrder.py
 
     """
-    TC = nx.DiGraph()
-    TC.add_nodes_from(G.nodes())
-    TC.add_edges_from(G.edges())
+    TC = G.copy()
     for v in G:
         TC.add_edges_from((v, u) for u in nx.dfs_preorder_nodes(G, source=v)
                           if v != u)
@@ -393,15 +540,24 @@ def transitive_reduction(G):
 
     """
     if not is_directed_acyclic_graph(G):
-        raise nx.NetworkXError(
-            "Transitive reduction only uniquely defined on directed acyclic graphs.")
+        msg = "Directed Acyclic Graph required for transitive_reduction"
+        raise nx.NetworkXError(msg)
     TR = nx.DiGraph()
     TR.add_nodes_from(G.nodes())
+    descendants = {}
+    # count before removing set stored in descendants
+    check_count = dict(G.in_degree)
     for u in G:
-        u_edges = set(G[u])
+        u_nbrs = set(G[u])
         for v in G[u]:
-            u_edges -= {y for x, y in nx.dfs_edges(G, v)}
-        TR.add_edges_from((u, v) for v in u_edges)
+            if v in u_nbrs:
+                if v not in descendants:
+                    descendants[v] = {y for x, y in nx.dfs_edges(G, v)}
+                u_nbrs -= descendants[v]
+            check_count[v] -= 1
+            if check_count[v] == 0:
+                del descendants[v]
+        TR.add_edges_from((u, v) for v in u_nbrs)
     return TR
 
 
@@ -462,7 +618,8 @@ def antichains(G):
 def dag_longest_path(G, weight='weight', default_weight=1):
     """Returns the longest path in a directed acyclic graph (DAG).
 
-    If `G` has edges with `weight` attribute the edge data are used as weight values.
+    If `G` has edges with `weight` attribute the edge data are used as
+    weight values.
 
     Parameters
     ----------
@@ -488,12 +645,16 @@ def dag_longest_path(G, weight='weight', default_weight=1):
     See also
     --------
     dag_longest_path_length
+
     """
+    if not G:
+        return []
     dist = {}  # stores {v : (length, u)}
     for v in nx.topological_sort(G):
         us = [(dist[u][0] + data.get(weight, default_weight), u)
               for u, data in G.pred[v].items()]
-        # Use the best predecessor if there is one and its distance is non-negative, otherwise terminate.
+        # Use the best predecessor if there is one and its distance is
+        # non-negative, otherwise terminate.
         maxu = max(us, key=lambda x: x[0]) if us else (0, v)
         dist[v] = maxu if maxu[0] >= 0 else (0, v)
     u = None
@@ -542,3 +703,119 @@ def dag_longest_path_length(G, weight='weight', default_weight=1):
         path_length += G[u][v].get(weight, default_weight)
 
     return path_length
+
+
+def root_to_leaf_paths(G):
+    """Yields root-to-leaf paths in a directed acyclic graph.
+
+    `G` must be a directed acyclic graph. If not, the behavior of this
+    function is undefined. A "root" in this graph is a node of in-degree
+    zero and a "leaf" a node of out-degree zero.
+
+    When invoked, this function iterates over each path from any root to
+    any leaf. A path is a list of nodes.
+
+    """
+    roots = (v for v, d in G.in_degree() if d == 0)
+    leaves = (v for v, d in G.out_degree() if d == 0)
+    all_paths = partial(nx.all_simple_paths, G)
+    # TODO In Python 3, this would be better as `yield from ...`.
+    return chaini(starmap(all_paths, product(roots, leaves)))
+
+
+@not_implemented_for('multigraph')
+@not_implemented_for('undirected')
+def dag_to_branching(G):
+    """Returns a branching representing all (overlapping) paths from
+    root nodes to leaf nodes in the given directed acyclic graph.
+
+    As described in :mod:`networkx.algorithms.tree.recognition`, a
+    *branching* is a directed forest in which each node has at most one
+    parent. In other words, a branching is a disjoint union of
+    *arborescences*. For this function, each node of in-degree zero in
+    `G` becomes a root of one of the arborescences, and there will be
+    one leaf node for each distinct path from that root to a leaf node
+    in `G`.
+
+    Each node `v` in `G` with *k* parents becomes *k* distinct nodes in
+    the returned branching, one for each parent, and the sub-DAG rooted
+    at `v` is duplicated for each copy. The algorithm then recurses on
+    the children of each copy of `v`.
+
+    Parameters
+    ----------
+    G : NetworkX graph
+        A directed acyclic graph.
+
+    Returns
+    -------
+    DiGraph
+        The branching in which there is a bijection between root-to-leaf
+        paths in `G` (in which multiple paths may share the same leaf)
+        and root-to-leaf paths in the branching (in which there is a
+        unique path from a root to a leaf).
+
+        Each node has an attribute 'source' whose value is the original
+        node to which this node corresponds. No other graph, node, or
+        edge attributes are copied into this new graph.
+
+    Raises
+    ------
+    NetworkXNotImplemented
+        If `G` is not directed, or if `G` is a multigraph.
+
+    HasACycle
+        If `G` is not acyclic.
+
+    Examples
+    --------
+    To examine which nodes in the returned branching were produced by
+    which original node in the directed acyclic graph, we can collect
+    the mapping from source node to new nodes into a dictionary. For
+    example, consider the directed diamond graph::
+
+        >>> from collections import defaultdict
+        >>> from operator import itemgetter
+        >>>
+        >>> G = nx.DiGraph(nx.utils.pairwise('abd'))
+        >>> G.add_edges_from(nx.utils.pairwise('acd'))
+        >>> B = nx.dag_to_branching(G)
+        >>>
+        >>> sources = defaultdict(set)
+        >>> for v, source in B.nodes(data='source'):
+        ...     sources[source].add(v)
+        >>> len(sources['a'])
+        1
+        >>> len(sources['d'])
+        2
+
+    To copy node attributes from the original graph to the new graph,
+    you can use a dictionary like the one constructed in the above
+    example::
+
+        >>> for source, nodes in sources.items():
+        ...     for v in nodes:
+        ...         B.node[v].update(G.node[source])
+
+    Notes
+    -----
+    This function is not idempotent in the sense that the node labels in
+    the returned branching may be uniquely generated each time the
+    function is invoked. In fact, the node labels may not be integers;
+    in order to relabel the nodes to be more readable, you can use the
+    :func:`networkx.convert_node_labels_to_integers` function.
+
+    The current implementation of this function uses
+    :func:`networkx.prefix_tree`, so it is subject to the limitations of
+    that function.
+
+    """
+    if has_cycle(G):
+        msg = 'dag_to_branching is only defined for acyclic graphs'
+        raise nx.HasACycle(msg)
+    paths = root_to_leaf_paths(G)
+    B, root = nx.prefix_tree(paths)
+    # Remove the synthetic `root` and `NIL` nodes in the prefix tree.
+    B.remove_node(root)
+    B.remove_node(NIL)
+    return B
