@@ -1,16 +1,20 @@
 # -*- coding: utf-8 -*-
 """Floyd-Warshall algorithm for shortest paths.
 """
-#    Copyright (C) 2004-2017 by
+#    Copyright (C) 2004-2018 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
+#
+# Authors: Aric Hagberg <aric.hagberg@gmail.com>
+#          Miguel Sozinho Ramalho <m.ramalho@fe.up.pt>
 import networkx as nx
-__author__ = """Aric Hagberg <aric.hagberg@gmail.com>"""
+
 __all__ = ['floyd_warshall',
            'floyd_warshall_predecessor_and_distance',
+           'reconstruct_path',
            'floyd_warshall_numpy']
 
 
@@ -76,6 +80,16 @@ def floyd_warshall_predecessor_and_distance(G, weight='weight'):
        Dictionaries, keyed by source and target, of predecessors and distances
        in the shortest path.
 
+    Examples
+    --------
+    >>> G = nx.DiGraph()
+    >>> G.add_weighted_edges_from([('s', 'u', 10), ('s', 'x', 5),
+    ...     ('u', 'v', 1), ('u', 'x', 2), ('v', 'y', 1), ('x', 'u', 3),
+    ...     ('x', 'v', 5), ('x', 'y', 2), ('y', 's', 7), ('y', 'v', 6)])
+    >>> predecessors, _ = nx.floyd_warshall_predecessor_and_distance(G)
+    >>> print(nx.reconstruct_path('s', 'v', predecessors))
+    ['s', 'x', 'u', 'v']
+
     Notes
     ------
     Floyd's algorithm is appropriate for finding shortest paths
@@ -117,6 +131,49 @@ def floyd_warshall_predecessor_and_distance(G, weight='weight'):
     return dict(pred), dict(dist)
 
 
+def reconstruct_path(source, target, predecessors):
+    """Reconstruct a path from source to target using the predecessors
+    dict as returned by floyd_warshall_predecessor_and_distance
+
+    Parameters
+    ----------
+    source : node
+       Starting node for path
+
+    target : node
+       Ending node for path
+
+    predecessors: dictionary
+       Dictionary, keyed by source and target, of predecessors in the
+       shortest path, as returned by floyd_warshall_predecessor_and_distance
+
+    Returns
+    -------
+    path : list
+       A list of nodes containing the shortest path from source to target
+
+       If source and target are the same, an empty list is returned
+
+    Notes
+    ------
+    This function is meant to give more applicability to the
+    floyd_warshall_predecessor_and_distance function
+
+    See Also
+    --------
+    floyd_warshall_predecessor_and_distance
+    """
+    if source == target:
+        return []
+    prev = predecessors[source]
+    curr = prev[target]
+    path = [target, curr]
+    while curr != source:
+        curr = prev[curr]
+        path.append(curr)
+    return list(reversed(path))
+
+
 def floyd_warshall(G, weight='weight'):
     """Find all-pairs shortest path lengths using Floyd's algorithm.
 
