@@ -1,7 +1,9 @@
 from nose.tools import *
 
 import networkx as nx
+
 edge_dfs = nx.algorithms.edge_dfs
+
 FORWARD = nx.algorithms.edgedfs.FORWARD
 REVERSE = nx.algorithms.edgedfs.REVERSE
 
@@ -38,6 +40,24 @@ class TestEdgeDFS(object):
         x_ = [(0, 1), (1, 0), (2, 1), (3, 1)]
         assert_equal(x, x_)
 
+    def test_digraph_orientation_invalid(self):
+        G = nx.DiGraph(self.edges)
+        edge_iterator = edge_dfs(G, self.nodes, orientation='hello')
+        assert_raises(nx.NetworkXError, list, edge_iterator)
+
+    def test_digraph_orientation_none(self):
+        G = nx.DiGraph(self.edges)
+        x = list(edge_dfs(G, self.nodes, orientation=None))
+        x_ = [(0, 1), (1, 0), (2, 1), (3, 1)]
+        assert_equal(x, x_)
+
+    def test_digraph_orientation_original(self):
+        G = nx.DiGraph(self.edges)
+        x = list(edge_dfs(G, self.nodes, orientation='original'))
+        x_ = [(0, 1, FORWARD), (1, 0, FORWARD),
+              (2, 1, FORWARD), (3, 1, FORWARD)]
+        assert_equal(x, x_)
+
     def test_digraph2(self):
         G = nx.DiGraph()
         nx.add_path(G, range(4))
