@@ -42,16 +42,14 @@ def from_agraph(A, create_using=None):
     A : PyGraphviz AGraph
       A graph created with PyGraphviz
 
-    create_using : NetworkX graph class instance
-      The output is created using the given graph class instance
+    create_using : NetworkX graph constructor, optional (default=nx.Graph)
+       Graph type to create. If graph instance, then cleared before populated.
 
     Examples
     --------
     >>> K5 = nx.complete_graph(5)
     >>> A = nx.nx_agraph.to_agraph(K5)
     >>> G = nx.nx_agraph.from_agraph(A)
-    >>> G = nx.nx_agraph.from_agraph(A)
-
 
     Notes
     -----
@@ -69,14 +67,14 @@ def from_agraph(A, create_using=None):
     if create_using is None:
         if A.is_directed():
             if A.is_strict():
-                create_using = nx.DiGraph()
+                create_using = nx.DiGraph
             else:
-                create_using = nx.MultiDiGraph()
+                create_using = nx.MultiDiGraph
         else:
             if A.is_strict():
-                create_using = nx.Graph()
+                create_using = nx.Graph
             else:
-                create_using = nx.MultiGraph()
+                create_using = nx.MultiGraph
 
     # assign defaults
     N = nx.empty_graph(0, create_using)
@@ -145,7 +143,8 @@ def to_agraph(N):
     A.node_attr.update(N.graph.get('node', {}))
     A.edge_attr.update(N.graph.get('edge', {}))
 
-    A.graph_attr.update(N.graph)
+    A.graph_attr.update((k, v) for k, v in N.graph.items()
+                        if k not in ('graph', 'node', 'edge'))
 
     # add nodes
     for n, nodedata in N.nodes(data=True):
@@ -157,7 +156,8 @@ def to_agraph(N):
     # loop over edges
     if N.is_multigraph():
         for u, v, key, edgedata in N.edges(data=True, keys=True):
-            str_edgedata = {k: str(v) for k, v in edgedata.items() if k != 'key'}
+            str_edgedata = {k: str(v) for k, v in edgedata.items()
+                            if k != 'key'}
             A.add_edge(u, v, key=str(key))
             if edgedata is not None:
                 a = A.get_edge(u, v)
@@ -266,6 +266,18 @@ def pygraphviz_layout(G, prog='neato', root=None, args=''):
     >>> pos = nx.nx_agraph.graphviz_layout(G)
     >>> pos = nx.nx_agraph.graphviz_layout(G, prog='dot')
 
+    Notes
+    -----
+    If you use complex node objects, they may have the same string
+    representation and GraphViz could treat them as the same node.
+    The layout may assign both nodes a single location. See Issue #1568
+    If this occurs in your case, consider relabeling the nodes just
+    for the layout computation using something similar to:
+
+        H = nx.convert_node_labels_to_integers(G, label_attribute='node_label')
+        H_layout = nx.nx_agraph.pygraphviz_layout(G, prog='dot')
+        G_layout = {H.nodes[n]['node_label']: p for n, p in H_layout.items()}
+
     """
     try:
         import pygraphviz
@@ -288,7 +300,7 @@ def pygraphviz_layout(G, prog='neato', root=None, args=''):
     return node_pos
 
 
-@nx.utils.open_file(5, 'w')
+@nx.utils.open_file(5, 'w+b')
 def view_pygraphviz(G, edgelabel=None, prog='dot', args='',
                     suffix='', path=None):
     """Views the graph G using the specified layout algorithm.
