@@ -8,7 +8,7 @@ Do not add this file to the repository.
 import datetime
 
 version = '2.2'
-date = 'Thu Feb 20 18:52:15 2020'
+date = 'Thu Jul  4 19:48:06 2019'
 
 # Was NetworkX built from a development version? If so, remember that the major
 # and minor versions reference the "target" (rather than "current") release.
@@ -18,7 +18,7 @@ dev = False
 version_info = ('networkx', '2', '2', None)
 
 # Format: a 'datetime.datetime' instance
-date_info = datetime.datetime(2020, 2, 20, 18, 52, 15)
+date_info = datetime.datetime(2019, 7, 4, 19, 48, 6)
 
 # Format: (vcs, vcs_tuple)
 vcs_info = (None, (None, None))
